/**
 * Sieve of Eratosthenes
 * Computes and prints the prime numbers up to MAXNUM.
 * This version uses bit-array for maximal space efficiency.
 */

#include <pbstdlib.h>

/* Define the maximal number, if not done so on the command line.
 * MAXNUM/8 bytes will be consumed, so be careful not to overflow the stack.
 */
#ifndef MAXNUM
# define MAXNUM          320
#endif

/* Choose appropriate data type to represent a value */
#if MAXNUM < 256
typedef uint8_t  index_t;
#else
typedef uint16_t index_t;
#endif

/* returns 1 if the given value 'v' is known NOT be to a prime */
static inline uint8_t getv(uint8_t *sieve, index_t v)
{
    return (sieve[v >> 3] >> (v & 0x07)) & 1;
}

/* set a flag for the given value 'v' indicating it is NOT a prime */
static inline void setv(uint8_t *sieve, index_t v)
{
    sieve[v >> 3] |= 1 << (v & 0x07);
}

int main()
{
    /* Sieve bit array. Each BIT represents one value
     * from 0 to MAXNUM. */
    uint8_t sieve[MAXNUM/8];
    
    /* clear the array to zero */
    for (uint8_t i = 0; i < MAXNUM/8; ++i)
        sieve[i] = 0;
    
    /* start from 2, the first prime */
    for (index_t i = 2; ; ++i) {
        /* find the first prime in the sieve, i.e. the first
         * number without a flag set */
        for (; i < MAXNUM; ++i) {
            if (getv(sieve, i) == 0) {
                /* ok, it's prime; 
                 * Print LO byte to port 0, and HI byte to port 1. */
                __output(0, i);
                __output(1, i >> 8);
                break;
            }
        }
        /* did we find any? */
        if (i == MAXNUM)
            /* no, stop */
            return 0;
        
        /* Mark all its multiplies up to MAXNUM not be a prime.
         * Here we have to use 16 bit integers pretty much
         * always because of the condition test.
         */
        for (uint16_t k = 2*i; k < MAXNUM; k += i) {
            setv(sieve, k);
        }
    }
    
    return 0;
}
