library ieee;
use ieee.std_logic_1164.all;

--! Package declares PacoBlaze processor component

package comp_pblaze is

-- Corresponding Verilog module:
-- 
-- module `PACOBLAZE(
-- 	address, instruction,
-- 	port_id,
-- 	write_strobe, out_port,
-- 	read_strobe, in_port,
-- 	interrupt,
-- `ifdef HAS_INTERRUPT_ACK
-- 	interrupt_ack,
-- `endif
-- 	reset, clk
-- );
-- output [`code_depth-1:0] address; ///< Address output
-- input [`code_width-1:0] instruction; ///< Instruction input
-- output [`port_depth-1:0] port_id; ///< Port address
-- output write_strobe; ///< Port output strobe
-- output [`port_width-1:0] out_port; ///< Port output
-- output read_strobe; ///< Port input strobe
-- input [`port_width-1:0] in_port; ///< Port input
-- input interrupt; ///< Interrupt request
-- `ifdef HAS_INTERRUPT_ACK
-- output interrupt_ack; ///< Interrupt acknowledge (output)
-- `endif
-- input reset; ///< Reset input
-- input clk; ///< Clock input

--! Width of pblaze instr address bus
constant PB3_CODE_DEPTH : integer := 10;

--! Width of pblaze instruction
constant PB3_CODE_WIDTH : integer := 18;

--! Width of port_id address
constant PB3_PORT_DEPTH : integer := 8;

--! Width of inp/outp port data
constant PB3_PORT_WIDTH : integer := 8;

--! register width
constant PB3_REGISTER_WIDTH : integer := PB3_PORT_WIDTH;

--! pacoblaze3 processor, compatible with Xilinx PicoBlaze3 KCPSM3
--! The actual core is in verilog code.
-- component pacoblaze3 is
-- port (
--     address	: out std_logic_vector(PB3_CODE_DEPTH-1 downto 0);	--! instruction address output
--     instruction	: in  std_logic_vector(PB3_CODE_WIDTH-1 downto 0);	--! instruction input
--     port_id	: out std_logic_vector(PB3_PORT_DEPTH-1 downto 0);	--! port address output
--     write_strobe : out std_logic;					--! write on a port
--     out_port	: out std_logic_vector(PB3_PORT_WIDTH-1 downto 0);	--! output data to write on port
--     read_strobe	: out std_logic;					--! read from a port
--     in_port	: in std_logic_vector(PB3_PORT_WIDTH-1 downto 0);	--! input data from a port
--     interrupt	: in std_logic;						--! interrupt request
--     interrupt_ack : out std_logic;					--! interrupt ack
--     reset	: in std_logic;						--! reset signal
--     clk		: in std_logic;						--! clock signal
--     --! debug ports
--     dbg_rslt_we : out std_logic;                                        --! writing result into rX
--     dbg_rslt_dt : out std_logic_vector(PB3_REGISTER_WIDTH-1 downto 0)  --! the result value
-- );
-- end component pacoblaze3;

--! pacoblaze3a processor, compatible with Xilinx PicoBlaze3 KCPSM3
--! The actual core is in verilog code.
--! It has the ADDN instruction and reg+ofs addressing mode.
component pacoblaze3a is
port (
    address     : out std_logic_vector(PB3_CODE_DEPTH-1 downto 0);      --! instruction address output
    instruction : in  std_logic_vector(PB3_CODE_WIDTH-1 downto 0);      --! instruction input
    port_id     : out std_logic_vector(PB3_PORT_DEPTH-1 downto 0);      --! port address output
    write_strobe : out std_logic;                                       --! write on a port
    out_port    : out std_logic_vector(PB3_PORT_WIDTH-1 downto 0);      --! output data to write on port
    read_strobe : out std_logic;                                        --! read from a port
    in_port     : in std_logic_vector(PB3_PORT_WIDTH-1 downto 0);       --! input data from a port
    port_busy   : in std_logic;                                         --! port is busy
    interrupt   : in std_logic;                                         --! interrupt request
    interrupt_ack : out std_logic;                                      --! interrupt ack
    reset       : in std_logic;                                         --! reset signal
    clk         : in std_logic;                                         --! clock signal
    --! debug ports
    dbg_rslt_we : out std_logic;                                        --! writing result into rX
    dbg_rslt_dt : out std_logic_vector(PB3_REGISTER_WIDTH-1 downto 0)  --! the result value
);
end component pacoblaze3a;


-- --! Width of pblaze instr address bus
-- constant PB3E_CODE_DEPTH : integer := 10;
-- 
-- --! Width of pblaze instruction
-- constant PB3E_CODE_WIDTH : integer := 18;
-- 
-- --! Width of port_id address
-- constant PB3E_PORT_DEPTH : integer := 16;
-- 
-- --! Width of inp/outp port data
-- constant PB3E_PORT_WIDTH : integer := 16;
-- 
-- --! register width
-- constant PB3E_REGISTER_WIDTH : integer := 16;
-- 
-- --! pacoblaze3e processor with 16-bit datapaths, port_busy signal and new 16b support instructions (loadhi, loadse).
-- --! The actual core is in verilog code.
-- component pacoblaze3e is
-- port (
--     address     : out std_logic_vector(PB3E_CODE_DEPTH-1 downto 0);      --! instruction address output
--     instruction : in  std_logic_vector(PB3E_CODE_WIDTH-1 downto 0);      --! instruction input
--     port_id     : out std_logic_vector(PB3E_PORT_DEPTH-1 downto 0);      --! port address output
--     write_strobe : out std_logic;                                       --! write on a port
--     out_port    : out std_logic_vector(PB3E_PORT_WIDTH-1 downto 0);      --! output data to write on port
--     read_strobe : out std_logic;                                        --! read from a port
--     in_port     : in std_logic_vector(PB3E_PORT_WIDTH-1 downto 0);       --! input data from a port
--     port_busy   : in std_logic;                                         --! port is busy
--     interrupt   : in std_logic;                                         --! interrupt request
--     interrupt_ack : out std_logic;                                      --! interrupt ack
--     reset       : in std_logic;                                         --! reset signal
--     clk         : in std_logic;                                          --! clock signal
--     --! debug ports
--     dbg_rslt_we : out std_logic;                                        --! writing result into rX
--     dbg_rslt_dt : out std_logic_vector(PB3E_REGISTER_WIDTH-1 downto 0)  --! the result value
-- );
-- end component pacoblaze3e;


end package comp_pblaze;


package body comp_pblaze is
end package body comp_pblaze;
