library std;
use std.textio.all;
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library std_developerskit;
use std_developerskit.std_iopak.all;

library pblib;
use pblib.text_utils.all;


package pblaze3_dbg is

-- instruction opcode to string
function pb3instr2string(instr: std_logic_vector(17 downto 0)) return string;

end package pblaze3_dbg;


package body pblaze3_dbg is

-- picoblaze first opcode field
subtype pblaze_opcode1 is std_logic_vector(5 downto 0);

constant ADD_SX_KK      : pblaze_opcode1 := "011000";
constant ADD_SX_SY      : pblaze_opcode1 := "011001";
constant ADDCY_SX_KK    : pblaze_opcode1 := "011010";
constant ADDCY_SX_SY    : pblaze_opcode1 := "011011";
constant ADDN_SX_KK     : pblaze_opcode1 := "010000";            -- PB3A
constant ADDN_SX_SY     : pblaze_opcode1 := "010001";           -- PB3A
constant AND_SX_KK      : pblaze_opcode1 := "001010";
constant AND_SX_SY      : pblaze_opcode1 := "001011";
constant CALL           : pblaze_opcode1 := "110000";
constant CALL_X         : pblaze_opcode1 := "110001";
constant CMP_SX_KK      : pblaze_opcode1 := "010100";
constant CMP_SX_SY      : pblaze_opcode1 := "010101";
constant ENDIS_INT      : pblaze_opcode1 := "111100";
constant FETCH_SX_SS    : pblaze_opcode1 := "000110";
constant FETCH_SX_SY    : pblaze_opcode1 := "000111";
constant INPUT_SX_SY    : pblaze_opcode1 := "000101";
constant INPUT_SX_PP    : pblaze_opcode1 := "000100";
constant JUMP           : pblaze_opcode1 := "110100";
constant JUMP_X         : pblaze_opcode1 := "110101";
constant LOAD_SX_KK     : pblaze_opcode1 := "000000";
constant LOAD_SX_SY     : pblaze_opcode1 := "000001";
constant LOADHI_SX_KK   : pblaze_opcode1 := "000010";
constant TWIST_SX_SY    : pblaze_opcode1 := "000011";
constant LOADSE_SX_KK   : pblaze_opcode1 := "001000";
constant LOADSE_SX_SY   : pblaze_opcode1 := "001001";
constant OR_SX_KK       : pblaze_opcode1 := "001100";
constant OR_SX_SY       : pblaze_opcode1 := "001101";
constant OUTPUT_SX_SY   : pblaze_opcode1 := "101101";
constant OUTPUT_SX_PP   : pblaze_opcode1 := "101100";
constant PRETURN        : pblaze_opcode1 := "101010";
constant PRETURN_X      : pblaze_opcode1 := "101011";
constant PRETURNI       : pblaze_opcode1 := "111000";
constant ROTATE         : pblaze_opcode1 := "100000";
constant STORE_SX_SS    : pblaze_opcode1 := "101110";
constant STORE_SX_SY    : pblaze_opcode1 := "101111";
constant SUB_SX_KK      : pblaze_opcode1 := "011100";
constant SUB_SX_SY      : pblaze_opcode1 := "011101";
constant SUBCY_SX_KK    : pblaze_opcode1 := "011110";
constant SUBCY_SX_SY    : pblaze_opcode1 := "011111";
constant TEST_SX_KK     : pblaze_opcode1 := "010010";
constant TEST_SX_SY     : pblaze_opcode1 := "010011";
constant XOR_SX_KK      : pblaze_opcode1 := "001110";
constant XOR_SX_SY      : pblaze_opcode1 := "001111";

-- picoblaze rotate opcode
subtype pblaze_rotcode is std_logic_vector(3 downto 0);

constant RL_SX          : pblaze_rotcode := "0010";
constant RR_SX          : pblaze_rotcode := "1100";
constant SL0_SX         : pblaze_rotcode := "0110";
constant SL1_SX         : pblaze_rotcode := "0111";
constant SLA_SX         : pblaze_rotcode := "0000";
constant SLX_SX         : pblaze_rotcode := "0100";
constant SR0_SX         : pblaze_rotcode := "1110";
constant SR1_SX         : pblaze_rotcode := "1111";
constant SRA_SX         : pblaze_rotcode := "1000";
constant SRX_SX         : pblaze_rotcode := "1010";

--constant MAXSTRL : integer := 256;

-- instruction opcode to string
function pb3instr2string(instr: std_logic_vector(17 downto 0)) return string is
variable op1 : pblaze_opcode1;
variable sx, sy: std_logic_vector(3 downto 0);
variable op2 : std_logic_vector(1 downto 0);
variable op2_fl : string(1 to 2);
variable int_endi : string(1 to 10);
variable kk : std_logic_vector(7 downto 0);
variable aa : std_logic_vector(9 downto 0);
variable rotc : pblaze_rotcode;
variable str : string(1 to MAXSTRLEN);
begin
    op1 := instr(17 downto 12);
    op2 := instr(11 downto 10);
--     sx := to_integer(unsigned(instr(11 downto 8)));
--     sy := to_integer(unsigned(instr( 7 downto 4)));
    sx := instr(11 downto 8);
    sy := instr( 7 downto 4);
    kk := instr(7 downto 0);
    aa := instr(9 downto 0);
    rotc := instr(3 downto 0);
    str(1) := NUL;
    
    case op2 is
        when "00" => op2_fl := " Z";
        when "01" => op2_fl := "NZ";
        when "10" => op2_fl := " C";
        when "11" => op2_fl := "NC";
        when others => op2_fl := "??";
    end case;
    
    int_endi := "DISABLE" & NUL & "  ";
    if instr(0) = '1' then
        int_endi := "ENABLE" & NUL & "   ";
    end if;
    
    case op1 is
        when ADD_SX_KK          => fprint(str, "ADD    s%s, 0x%s", tosth(sx), tosth(kk));
        when ADD_SX_SY          => fprint(str, "ADD    s%s, s%s", tosth(sx), tosth(sy));
        when ADDCY_SX_KK        => fprint(str, "ADDCY  s%s, 0x%s", tosth(sx), tosth(kk));
        when ADDCY_SX_SY        => fprint(str, "ADDCY  s%s, s%s", tosth(sx), tosth(sy));
        when ADDN_SX_KK         => fprint(str, "ADDN   s%s, 0x%s", tosth(sx), tosth(kk));
        when ADDN_SX_SY         => fprint(str, "ADDN   s%s, s%s", tosth(sx), tosth(sy));
        when AND_SX_KK          => fprint(str, "AND    s%s, 0x%s", tosth(sx), tosth(kk));
        when AND_SX_SY          => fprint(str, "AND    s%s, s%s", tosth(sx), tosth(sy));
        when CALL               => fprint(str, "CALL   0x%s", tosth(aa));
        when CALL_X             => fprint(str, "CALL   %s, 0x%s", op2_fl, tosth(aa));
        when CMP_SX_KK          => fprint(str, "CMP    s%s, 0x%s", tosth(sx), tosth(kk));
        when CMP_SX_SY          => fprint(str, "CMP    s%s, s%s", tosth(sx), tosth(sy));
        when ENDIS_INT          => fprint(str, "%s INTERRUPT", int_endi);
        when FETCH_SX_SS        => fprint(str, "FETCH  s%s, 0x%s", tosth(sx), tosth(kk));
        when FETCH_SX_SY        => fprint(str, "FETCH  s%s, (s%s), 0x%s", tosth(sx), tosth(sy), tosth(kk(3 downto 0)));
        when INPUT_SX_SY        => fprint(str, "INPUT  s%s, (s%s), 0x%s", tosth(sx), tosth(sy), tosth(kk(3 downto 0)));
        when INPUT_SX_PP        => fprint(str, "INPUT  s%s, 0x%s", tosth(sx), tosth(kk));
        when JUMP               => fprint(str, "JUMP   0x%s", tosth(aa));
        when JUMP_X             => fprint(str, "JUMP   %s, 0x%s", op2_fl, tosth(aa));
        when LOAD_SX_KK         => fprint(str, "LOAD   s%s, 0x%s", tosth(sx), tosth(kk));
        when LOAD_SX_SY         => fprint(str, "LOAD   s%s, s%s", tosth(sx), tosth(sy));
        when LOADHI_SX_KK       => fprint(str, "LOADHI s%s, 0x%s", tosth(sx), tosth(kk));
        when TWIST_SX_SY        => fprint(str, "TWIST  s%s, s%s", tosth(sx), tosth(sy));
        when LOADSE_SX_KK       => fprint(str, "LOADSE s%s, 0x%s", tosth(sx), tosth(kk));
        when LOADSE_SX_SY       => fprint(str, "LOADSE s%s, s%s", tosth(sx), tosth(sy));
        when OR_SX_KK           => fprint(str, "OR     s%s, 0x%s", tosth(sx), tosth(kk));
        when OR_SX_SY           => fprint(str, "OR     s%s, s%s", tosth(sx), tosth(sy));
        when OUTPUT_SX_SY       => fprint(str, "OUTPUT s%s, (s%s), 0x%s", tosth(sx), tosth(sy), tosth(kk(3 downto 0)));
        when OUTPUT_SX_PP       => fprint(str, "OUTPUT s%s, 0x%s", tosth(sx), tosth(kk));
        when PRETURN            => fprint(str, "RETURN");
        when PRETURN_X          => fprint(str, "RETURN %s", op2_fl);
        when PRETURNI           => fprint(str, "RETURNI %s", int_endi);
        when ROTATE             => 
            case rotc is
                when RL_SX      => fprint(str, "RL     s%s", tosth(sx));
                when RR_SX      => fprint(str, "RR     s%s", tosth(sx));
                when SL0_SX     => fprint(str, "SL0    s%s", tosth(sx));
                when SL1_SX     => fprint(str, "SL1    s%s", tosth(sx));
                when SLA_SX     => fprint(str, "SLA    s%s", tosth(sx));
                when SLX_SX     => fprint(str, "SLX    s%s", tosth(sx));
                when SR0_SX     => fprint(str, "SR0    s%s", tosth(sx));
                when SR1_SX     => fprint(str, "SR1    s%s", tosth(sx));
                when SRA_SX     => fprint(str, "SRA    s%s", tosth(sx));
                when SRX_SX     => fprint(str, "SRX    s%s", tosth(sx));
                when others     => fprint(str, "unknown rotate 0x%s", tosth(instr));
            end case;
        when STORE_SX_SS        => fprint(str, "STORE  s%s, 0x%s", tosth(sx), tosth(kk));
        when STORE_SX_SY        => fprint(str, "STORE  s%s, (s%s), 0x%s", tosth(sx), tosth(sy), tosth(kk(3 downto 0)));
        when SUB_SX_KK          => fprint(str, "SUB    s%s, 0x%s", tosth(sx), tosth(kk));
        when SUB_SX_SY          => fprint(str, "SUB    s%s, s%s", tosth(sx), tosth(sy));
        when SUBCY_SX_KK        => fprint(str, "SUBCY  s%s, 0x%s", tosth(sx), tosth(kk));
        when SUBCY_SX_SY        => fprint(str, "SUBCY  s%s, s%s", tosth(sx), tosth(sy));
        when TEST_SX_KK         => fprint(str, "TEST   s%s, 0x%s", tosth(sx), tosth(kk));
        when TEST_SX_SY         => fprint(str, "TEST   s%s, s%s", tosth(sx), tosth(sy));
        when XOR_SX_KK          => fprint(str, "XOR    s%s, 0x%s", tosth(sx), tosth(kk));
        when XOR_SX_SY          => fprint(str, "XOR    s%s, s%s", tosth(sx), tosth(sy));
        when others             => fprint(str, "unknown 0x%s", tosth(instr));
    end case;
    
    return str;
end function pb3instr2string;

end package body;
