library std;
use std.textio.all;
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library std_developerskit;
use std_developerskit.std_iopak.all;

library pblib;
use pblib.text_utils.all;


package pblaze6_dbg is

-- instruction opcode to string
function pb6instr2string(instr: std_logic_vector(17 downto 0)) return string;

end package pblaze6_dbg;


package body pblaze6_dbg is

-- picoblaze first opcode field
subtype pblaze_opcode1 is std_logic_vector(5 downto 0);

constant ADD_SX_KK      : pblaze_opcode1 := "010001";
constant ADD_SX_SY      : pblaze_opcode1 := "010000";
constant ADDCY_SX_KK    : pblaze_opcode1 := "010011";
constant ADDCY_SX_SY    : pblaze_opcode1 := "010010";
constant AND_SX_KK      : pblaze_opcode1 := "000011";
constant AND_SX_SY      : pblaze_opcode1 := "000010";
constant CALL           : pblaze_opcode1 := "100000";
constant CALL_Z         : pblaze_opcode1 := "110000";
constant CALL_NZ        : pblaze_opcode1 := "110100";
constant CALL_C         : pblaze_opcode1 := "111000";
constant CALL_NC        : pblaze_opcode1 := "111100";
constant CMP_SX_KK      : pblaze_opcode1 := "011101";
constant CMP_SX_SY      : pblaze_opcode1 := "011100";
constant ENDIS_INT      : pblaze_opcode1 := "101000";
constant FETCH_SX_SS    : pblaze_opcode1 := "001011";
constant FETCH_SX_SY    : pblaze_opcode1 := "001010";
constant INPUT_SX_SY    : pblaze_opcode1 := "001000";
constant INPUT_SX_PP    : pblaze_opcode1 := "001001";
constant JUMP           : pblaze_opcode1 := "100010";
constant JUMP_Z         : pblaze_opcode1 := "110010";
constant JUMP_NZ        : pblaze_opcode1 := "110110";
constant JUMP_C         : pblaze_opcode1 := "111010";
constant JUMP_NC        : pblaze_opcode1 := "111110";
constant LOAD_SX_KK     : pblaze_opcode1 := "000001";
constant LOAD_SX_SY     : pblaze_opcode1 := "000000";
constant OR_SX_KK       : pblaze_opcode1 := "000101";
constant OR_SX_SY       : pblaze_opcode1 := "000100";
constant OUTPUT_SX_SY   : pblaze_opcode1 := "101100";
constant OUTPUT_SX_PP   : pblaze_opcode1 := "101101";
constant PRETURN        : pblaze_opcode1 := "100101";
constant PRETURN_Z      : pblaze_opcode1 := "110001";
constant PRETURN_NZ     : pblaze_opcode1 := "110101";
constant PRETURN_C      : pblaze_opcode1 := "111001";
constant PRETURN_NC     : pblaze_opcode1 := "111101";
constant PRETURNI       : pblaze_opcode1 := "101001";
constant ROTATE         : pblaze_opcode1 := "010100";
constant STORE_SX_SS    : pblaze_opcode1 := "101111";
constant STORE_SX_SY    : pblaze_opcode1 := "101110";
constant SUB_SX_KK      : pblaze_opcode1 := "011001";
constant SUB_SX_SY      : pblaze_opcode1 := "011000";
constant SUBCY_SX_KK    : pblaze_opcode1 := "011011";
constant SUBCY_SX_SY    : pblaze_opcode1 := "011010";
constant TEST_SX_KK     : pblaze_opcode1 := "001101";
constant TEST_SX_SY     : pblaze_opcode1 := "001100";
constant XOR_SX_KK      : pblaze_opcode1 := "000111";
constant XOR_SX_SY      : pblaze_opcode1 := "000110";

-- PB6 new:
constant CMPCY_SX_KK    : pblaze_opcode1 := "011111";
constant CMPCY_SX_SY    : pblaze_opcode1 := "011110";
constant TESTCY_SX_KK   : pblaze_opcode1 := "001111";
constant TESTCY_SX_SY   : pblaze_opcode1 := "001110";
constant STAR_SX_SY     : pblaze_opcode1 := "010110";
constant LOADRET_SX_KK  : pblaze_opcode1 := "100001";
constant REGBANK        : pblaze_opcode1 := "110111";
constant JUMPAT_SX_SY   : pblaze_opcode1 := "100110";
constant CALLAT_SX_SY   : pblaze_opcode1 := "100100";
constant OUTPUTK        : pblaze_opcode1 := "101011";


-- picoblaze rotate opcode
subtype pblaze_rotcode is std_logic_vector(7 downto 0);

constant RL_SX          : pblaze_rotcode := "00000010";
constant RR_SX          : pblaze_rotcode := "00001100";
constant SL0_SX         : pblaze_rotcode := "00000110";
constant SL1_SX         : pblaze_rotcode := "00000111";
constant SLA_SX         : pblaze_rotcode := "00000000";
constant SLX_SX         : pblaze_rotcode := "00000100";
constant SR0_SX         : pblaze_rotcode := "00001110";
constant SR1_SX         : pblaze_rotcode := "00001111";
constant SRA_SX         : pblaze_rotcode := "00001000";
constant SRX_SX         : pblaze_rotcode := "00001010";
constant HWBUILD_SX     : pblaze_rotcode := "10000000";

--constant MAXSTRL : integer := 256;

-- instruction opcode to string
function pb6instr2string(instr: std_logic_vector(17 downto 0)) return string is
variable op1 : pblaze_opcode1;
variable sx, sy: std_logic_vector(3 downto 0);
variable op2 : std_logic_vector(1 downto 0);
variable op2_fl : string(1 to 2);
variable int_endi : string(1 to 10);
variable kk : std_logic_vector(7 downto 0);
variable kk11d4 : std_logic_vector(7 downto 0);
variable aa : std_logic_vector(11 downto 0);
variable rotc : pblaze_rotcode;
variable str : string(1 to MAXSTRLEN);
variable rbank : string(1 to 2);
begin
    op1 := instr(17 downto 12);
    op2 := instr(15 downto 14);     -- flags
    sx := instr(11 downto 8);
    sy := instr( 7 downto 4);
    kk := instr(7 downto 0);
    kk11d4 := instr(11 downto 4);
    aa := instr(11 downto 0);
    rotc := instr(7 downto 0);
    str(1) := NUL;
    
    case op2 is
        when "00" => op2_fl := " Z";
        when "01" => op2_fl := "NZ";
        when "10" => op2_fl := " C";
        when "11" => op2_fl := "NC";
        when others => op2_fl := "??";
    end case;
    
    int_endi := "DISABLE" & NUL & "  ";
    if instr(0) = '1' then
        int_endi := "ENABLE" & NUL & "   ";
    end if;

    rbank := "A" & NUL;
    if instr(0) = '1' then
        rbank := "B" & NUL;
    end if;
    
    case op1 is
        when ADD_SX_KK          => fprint(str, "ADD    s%s, 0x%s", tosth(sx), tosth(kk));
        when ADD_SX_SY          => fprint(str, "ADD    s%s, s%s", tosth(sx), tosth(sy));
        when ADDCY_SX_KK        => fprint(str, "ADDCY  s%s, 0x%s", tosth(sx), tosth(kk));
        when ADDCY_SX_SY        => fprint(str, "ADDCY  s%s, s%s", tosth(sx), tosth(sy));
        when AND_SX_KK          => fprint(str, "AND    s%s, 0x%s", tosth(sx), tosth(kk));
        when AND_SX_SY          => fprint(str, "AND    s%s, s%s", tosth(sx), tosth(sy));
        when CALL               => fprint(str, "CALL   0x%s", tosth(aa));
        when CALL_Z | CALL_NZ | CALL_C | CALL_NC
                                => fprint(str, "CALL   %s, 0x%s", op2_fl, tosth(aa));
        when CALLAT_SX_SY       => fprint(str, "CALLAT s%s, s%s", tosth(sx), tosth(sy));
        when CMP_SX_KK          => fprint(str, "CMP    s%s, 0x%s", tosth(sx), tosth(kk));
        when CMP_SX_SY          => fprint(str, "CMP    s%s, s%s", tosth(sx), tosth(sy));
        when CMPCY_SX_KK        => fprint(str, "CMPCY  s%s, 0x%s", tosth(sx), tosth(kk));
        when CMPCY_SX_SY        => fprint(str, "CMPCY  s%s, s%s", tosth(sx), tosth(sy));
        when ENDIS_INT          => fprint(str, "%s INTERRUPT", int_endi);
        when FETCH_SX_SS        => fprint(str, "FETCH  s%s, 0x%s", tosth(sx), tosth(kk));
        when FETCH_SX_SY        => fprint(str, "FETCH  s%s, (s%s), 0x%s", tosth(sx), tosth(sy), tosth(kk(3 downto 0)));
        when INPUT_SX_SY        => fprint(str, "INPUT  s%s, (s%s), 0x%s", tosth(sx), tosth(sy), tosth(kk(3 downto 0)));
        when INPUT_SX_PP        => fprint(str, "INPUT  s%s, 0x%s", tosth(sx), tosth(kk));
        when JUMP               => fprint(str, "JUMP   0x%s", tosth(aa));
        when JUMP_Z | JUMP_NZ | JUMP_C | JUMP_NC
                                => fprint(str, "JUMP   %s, 0x%s", op2_fl, tosth(aa));
        when JUMPAT_SX_SY       => fprint(str, "JUMPAT s%s, s%s", tosth(sx), tosth(sy));
        when LOAD_SX_KK         => fprint(str, "LOAD   s%s, 0x%s", tosth(sx), tosth(kk));
        when LOAD_SX_SY         => fprint(str, "LOAD   s%s, s%s", tosth(sx), tosth(sy));
        when LOADRET_SX_KK      => fprint(str, "LOADRET s%s, 0x%s", tosth(sx), tosth(kk));
        when OR_SX_KK           => fprint(str, "OR     s%s, 0x%s", tosth(sx), tosth(kk));
        when OR_SX_SY           => fprint(str, "OR     s%s, s%s", tosth(sx), tosth(sy));
        when OUTPUT_SX_SY       => fprint(str, "OUTPUT s%s, (s%s), 0x%s", tosth(sx), tosth(sy), tosth(kk(3 downto 0)));
        when OUTPUT_SX_PP       => fprint(str, "OUTPUT s%s, 0x%s", tosth(sx), tosth(kk));
        when OUTPUTK            => fprint(str, "OUTPUTK 0x%s, 0x%s", tosth(kk11d4), tosth(kk(3 downto 0)));
        when PRETURN            => fprint(str, "RETURN");
        when PRETURN_Z | PRETURN_NZ | PRETURN_C | PRETURN_NC
                                => fprint(str, "RETURN %s", op2_fl);
        when PRETURNI           => fprint(str, "RETURNI %s", int_endi);
        when REGBANK            => fprint(str, "REGBANK %s", rbank);
        when ROTATE             => 
            case rotc is
                when RL_SX      => fprint(str, "RL     s%s", tosth(sx));
                when RR_SX      => fprint(str, "RR     s%s", tosth(sx));
                when SL0_SX     => fprint(str, "SL0    s%s", tosth(sx));
                when SL1_SX     => fprint(str, "SL1    s%s", tosth(sx));
                when SLA_SX     => fprint(str, "SLA    s%s", tosth(sx));
                when SLX_SX     => fprint(str, "SLX    s%s", tosth(sx));
                when SR0_SX     => fprint(str, "SR0    s%s", tosth(sx));
                when SR1_SX     => fprint(str, "SR1    s%s", tosth(sx));
                when SRA_SX     => fprint(str, "SRA    s%s", tosth(sx));
                when SRX_SX     => fprint(str, "SRX    s%s", tosth(sx));
                when HWBUILD_SX => fprint(str, "HWBUILD s%s", tosth(sx));
                when others     => fprint(str, "unknown rotate 0x%s", tosth(instr));
            end case;
        when STAR_SX_SY         => fprint(str, "STAR   s%s, s%s", tosth(sx), tosth(sy));
        when STORE_SX_SS        => fprint(str, "STORE  s%s, 0x%s", tosth(sx), tosth(kk));
        when STORE_SX_SY        => fprint(str, "STORE  s%s, (s%s), 0x%s", tosth(sx), tosth(sy), tosth(kk(3 downto 0)));
        when SUB_SX_KK          => fprint(str, "SUB    s%s, 0x%s", tosth(sx), tosth(kk));
        when SUB_SX_SY          => fprint(str, "SUB    s%s, s%s", tosth(sx), tosth(sy));
        when SUBCY_SX_KK        => fprint(str, "SUBCY  s%s, 0x%s", tosth(sx), tosth(kk));
        when SUBCY_SX_SY        => fprint(str, "SUBCY  s%s, s%s", tosth(sx), tosth(sy));
        when TEST_SX_KK         => fprint(str, "TEST   s%s, 0x%s", tosth(sx), tosth(kk));
        when TEST_SX_SY         => fprint(str, "TEST   s%s, s%s", tosth(sx), tosth(sy));
        when TESTCY_SX_KK       => fprint(str, "TESTCY s%s, 0x%s", tosth(sx), tosth(kk));
        when TESTCY_SX_SY       => fprint(str, "TESTCY s%s, s%s", tosth(sx), tosth(sy));
        when XOR_SX_KK          => fprint(str, "XOR    s%s, 0x%s", tosth(sx), tosth(kk));
        when XOR_SX_SY          => fprint(str, "XOR    s%s, s%s", tosth(sx), tosth(sy));
        when others             => fprint(str, "unknown 0x%s", tosth(instr));
    end case;
    
    return str;
end function pb6instr2string;

end package body;
