
use std.textio.all;

library ieee;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;

library std_developerskit;
use std_developerskit.std_iopak.all;


package text_utils is

constant MAXSTRLEN : positive := 1024;

function char2int(char : character) return integer;

function hexstr2int(str : string) return integer;

function decstr2int(str : string) return integer;

function hexstr2std_lv(str : string; vector_length : natural) return std_logic_vector;

function trim(raw_data : in string) return string;
function trim (raw_data : in string; length : in natural ) return string;

procedure print_string(file output : text; str : string );

procedure print_empty (file output : text);

subtype nibble is std_logic_vector(3 downto 0);

function tohex(n:nibble) return character;
function tosth(v:std_logic_vector) return string;

-- IN STD_IOPAK: return nul-terminated string length
-- function strlen(s : string) return integer;

-- IN STD_IOPAK: concat nul-terminated strings
-- procedure strcat(s1 : inout string; s2 : string);

-- IN STD_IOPAK: copy s2 into s1
-- procedure strcpy(s1 : out string; s2 : string);

-- IN STD_IOPAK: formatted print
-- procedure sprintf(rsltstr : out string; frm : string; 
--                     arg1 : string := "";
--                     arg2 : string := "";
--                     arg3 : string := "";
--                     arg4 : string := "";
--                     arg5 : string := "";
--                     arg6 : string := "";
--                     arg7 : string := "";
--                     arg8 : string := "";
--                     arg9 : string := "";
--                     arg10 : string := "");

-- formatted print to std output
procedure printf(frm : string; 
                    arg1 : string := "";
                    arg2 : string := "";
                    arg3 : string := "";
                    arg4 : string := "";
                    arg5 : string := "";
                    arg6 : string := "";
                    arg7 : string := "";
                    arg8 : string := "";
                    arg9 : string := "";
                    arg10 : string := "");

--! character substitution in string
function strsubst(s : in string; oldch : in character; newch : in character) return string;

end package text_utils;

--##############################################################################

package body text_utils is

function char2int(char : character) return integer is
  variable result : integer;
begin
  case char is
    when '0' => result := 0;
    when '1' => result := 1;
    when '2' => result := 2;
    when '3' => result := 3;
    when '4' => result := 4;
    when '5' => result := 5;
    when '6' => result := 6;
    when '7' => result := 7;
    when '8' => result := 8;
    when '9' => result := 9;
    when 'a' | 'A' => result := 10;
    when 'b' | 'B' => result := 11;
    when 'c' | 'C' => result := 12;
    when 'd' | 'D' => result := 13;
    when 'e' | 'E' => result := 14;
    when 'f' | 'F' => result := 15;
    when others => result := -1;
  end case;
  return result;
end function char2int;

--------------------------------------------------------------------------------

function basestr2int(str : string; base : positive) return integer is
  variable digit : integer;
  variable result : integer;
begin
  result := 0;
  -- characters are processed from left (most significant) to right (least significant)
  -- (assuming that the string is left-aligned (e.g. "12   ", "345  ", "007  " etc.)
  for i in str'low to str'high loop
    -- 0 - 15 for "0" - "F"; -1 for others 
    digit := char2int(str(i));
    if (digit < 0) then
      if (i = str'low) then
        result := -1;
      end if;
      exit;
    end if;    
    -- result is shifted by 4 to the left, the value of the current character is joined to the right
    result := result * base + digit;
  end loop;
  return result;
end function basestr2int; 

--------------------------------------------------------------------------------

function hexstr2int(str : string) return integer is
begin
  return basestr2int(str, 16);
end function hexstr2int;

--------------------------------------------------------------------------------

function decstr2int(str : string) return integer is
begin
  return basestr2int(str, 10);
end function decstr2int;

--------------------------------------------------------------------------------

function hexstr2std_lv(str : string; vector_length : natural) return std_logic_vector is
  variable digit : integer;
  variable result : std_logic_vector(vector_length - 1 downto 0);
begin
  result := (others => '0');
  -- characters are processed from left (most significant) to right (least significant)
  -- (assuming that the string is left-aligned (e.g. "12   ", "345  ", "007  " etc.)
  for i in str'low to str'high loop
    -- 0 - 15 for "0" - "F"; -1 for others 
    digit := char2int(str(i));
    exit when digit < 0;
    -- result is shifted by 4 to the left, the value of the current character is joined to the right
    result := result(vector_length - 5 downto 0) & conv_std_logic_vector(digit, 4);
  end loop;
  return result;
end function hexstr2std_lv;

--------------------------------------------------------------------------------

function trim( raw_data : in string) return string is
  variable trimmed_length : integer;
  variable heading_space : boolean;
  variable insert_space : boolean;
begin
  trimmed_length := 0;
  insert_space := false;
  heading_space := true;
  for raw_data_pointer in raw_data'left to raw_data'right loop
    if (raw_data(raw_data_pointer) = ' ') then
      if (not heading_space) then  
        insert_space := true;
      end if;  
    else
      heading_space := false;
      if (insert_space) then
        insert_space := false;
        trimmed_length := trimmed_length + 1;
      end if;    
      trimmed_length := trimmed_length + 1;
    end if;   
  end loop;
  
  return trim(raw_data, trimmed_length);
  
end function trim;

function trim( raw_data : in string; length : in natural ) return string is
  variable trimmed_data : string(1 to length);
  variable trimmed_data_pointer : positive;
  variable heading_space : boolean;
  variable insert_space : boolean;
begin
  if (length = 0) then
    return string'("");  
  end if;
  insert_space := false;
  heading_space := true;
  trimmed_data := (others => 'X');
  trimmed_data_pointer := trimmed_data'left;
  for raw_data_pointer in raw_data'left to raw_data'right loop
    if (raw_data(raw_data_pointer) = ' ') then
      if (not heading_space) then  
        insert_space := true;
      end if;  
    else
      heading_space := false;
      if (insert_space) then
        insert_space := false;
        trimmed_data(trimmed_data_pointer) := ' ';
        trimmed_data_pointer  := trimmed_data_pointer + 1;
        if (trimmed_data_pointer > trimmed_data'right) then
          exit;
        end if;  
      end if;    
      trimmed_data(trimmed_data_pointer) := raw_data(raw_data_pointer);
      trimmed_data_pointer  := trimmed_data_pointer + 1;
      if (trimmed_data_pointer > trimmed_data'right) then
        exit;
      end if;  
    end if;   
  end loop;  

  return trimmed_data;

end function trim;

procedure print_string(file output : text; str : string ) is
  variable buff : line;
begin
  write(buff, str);
  writeline(output, buff);  
end procedure print_string;

procedure print_empty (file output : text) is
  variable buff : line;
begin
  write(buff, string'(""));
  writeline(output, buff);  
end procedure print_empty;

--------------------------------------------------------------------------------

function tohex(n:nibble) return character is
begin
  case n is
  when "0000" => return('0');
  when "0001" => return('1');
  when "0010" => return('2');
  when "0011" => return('3');
  when "0100" => return('4');
  when "0101" => return('5');
  when "0110" => return('6');
  when "0111" => return('7');
  when "1000" => return('8');
  when "1001" => return('9');
  when "1010" => return('A');
  when "1011" => return('B');
  when "1100" => return('C');
  when "1101" => return('D');
  when "1110" => return('E');
  when "1111" => return('F');
  when others => return('X');
  end case;
end function;

function tosth(v:std_logic_vector) return string is
constant vlen : natural := v'length; --'
constant slen : natural := (vlen+3)/4;
variable vv : std_logic_vector(slen * 4 - 1 downto 0) := (others => '0');
variable s : string(1 to slen);
begin
  vv(v'length - 1 downto 0) := v;
  for i in slen downto 1 loop
    s(i) := tohex(vv(3 downto 0));
    vv(vv'left-4 downto 0) := vv(vv'left downto 4);
  end loop;
  return(s);
end function;

--------------------------------------------------------------------------------    
-- 
-- function strlen(s : string) return integer is
-- variable c : integer;
-- begin
--     c := 0;
--     for i in s'low to s'high loop
--         if s(i) = NUL then
--             return c;
--         end if;
--         c := c + 1;
--     end loop;
--     return c;
-- end strlen;
-- 
-- procedure strcat(s1 : inout string; s2 : string) is
-- variable s1_len : integer;
-- variable s2_len : integer;
-- begin
--     s1_len := strlen(s1);
--     s2_len := strlen(s2);
--     
--     s1((s1'low + s1_len) to (s1'low + s1_len + s2_len - 1)) := s2(s2'low to (s2'low + s2_len - 1));
--     s1((s1'low + s1_len + s2_len)) := NUL;
-- end strcat;
-- 
-- procedure strcpy(s1 : out string; s2 : string) is
-- variable s2_len : integer;
-- begin
--     s2_len := strlen(s2);
--     if (s1'length >= s2_len) then
--         s1(s1'low to (s1'low + s2_len - 1)) := s2(s2'low to (s2'low + s2_len - 1));
--         
--         if (s1'length > s2_len) then
--             s1(s1'low + s2_len) := NUL;
--         end if;
--     else
--         s1 := s2(s2'low to (s2'low + s1'length - 1));
--     end if;
-- end strcpy;
-- 
-- 
-- -- formatted print
-- procedure sprintf(rsltstr : out string; frm : string; 
--                     arg1 : string := "";
--                     arg2 : string := "";
--                     arg3 : string := "";
--                     arg4 : string := "";
--                     arg5 : string := "";
--                     arg6 : string := "";
--                     arg7 : string := "";
--                     arg8 : string := "";
--                     arg9 : string := "";
--                     arg10 : string := "") is
-- --type string_ptr is access string;
-- variable i, k, a : integer;
-- variable fnchar : character;         -- next character to frm(i)
-- variable str : string(1 to MAXSTRLEN);
-- --variable argx : string_ptr;
-- begin
--     i := frm'low;           -- rd pos in frm()
--     k := str'low;           -- wr pos in str()
--     a := 1;                 -- next argument index
--     str(k) := NUL;
--     
--     while i <= frm'high loop
--         -- get next character
--         if i < frm'high then
--             fnchar := frm(i+1);
--         else
--             fnchar := NUL;
--         end if;
--         
--         if (frm(i) = '%') and (fnchar = 's') then
--             -- argument substitution
--             case a is
--                 when 1 => strcat(str, arg1);
--                 when 2 => strcat(str, arg2);
--                 when 3 => strcat(str, arg3);
--                 when 4 => strcat(str, arg4);
--                 when 5 => strcat(str, arg5);
--                 when 6 => strcat(str, arg6);
--                 when 7 => strcat(str, arg7);
--                 when 8 => strcat(str, arg8);
--                 when 9 => strcat(str, arg9);
--                 when 10 => strcat(str, arg10);
--                 when others => null;
--             end case;
--             k := str'low + strlen(str);
--             i := i + 2;
--             a := a + 1;
--         elsif (frm(i) = '%') and (fnchar = '%') then
--             -- insert %
--             str(k) := '%';
--             k := k + 1;
--             i := i + 2;
--         else
--             -- default: copy the char
--             str(k) := frm(i);
--             k := k + 1;
--             i := i + 1;
--         end if;
--         
--         str(k) := NUL;
--     end loop;
--     
--     --rsltstr(rsltstr'low to (rsltstr'low + strlen(str))) 
--     strcpy(rsltstr, str);
-- end sprintf;

-- formatted print to std output
procedure printf(frm : string; 
                    arg1 : string := "";
                    arg2 : string := "";
                    arg3 : string := "";
                    arg4 : string := "";
                    arg5 : string := "";
                    arg6 : string := "";
                    arg7 : string := "";
                    arg8 : string := "";
                    arg9 : string := "";
                    arg10 : string := "") is
variable strbuf : string(1 to MAXSTRLEN);
begin
    fprint(strbuf, frm, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10);
    print_string(output, strbuf);
end procedure;

--! character substitution in string
function strsubst(s : in string; oldch : in character; newch : in character) return string is
variable os : string(1 to s'length);
begin
    for i in s'left to s'right loop
        if s(i) = oldch then
            os(i) := newch;
        else
            os(i) := s(i);
        end if;
    end loop;
    return os;
end function;

end package body;
