
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use std.textio.all;

library std_developerskit;
use std_developerskit.std_mempak.all;
use std_developerskit.std_iopak.all;

library pblib;
use pblib.comp_kcpsm3.all;
use pblib.comp_kcpsm6.all;
use pblib.comp_pblaze.all;
use pblib.pblaze3_dbg.all;
use pblib.pblaze6_dbg.all;
use pblib.text_utils.all;


entity pbsim is
generic (
    imem_fname: string;                         --! instruction memory, MEM file name
    pbver : string;                             --! picoblaze sub-arch version: kcpsm3, pb3a, kcpsm6
    portmon_fname: string := "portmon.txt"      --! output port monitor file
);
end entity pbsim;


architecture behav of pbsim is

-- address bit width in the program memory
function imem_a_width return integer is
begin
    if (pbver = "kcpsm3") or (pbver = "pb3a") then
        return 10;
    elsif (pbver = "kcpsm6") then
        return 12;
    else
        assert false report "Unknown pbver=" & pbver severity failure;
        return 0;
    end if;
end function;

constant zero_instruction: std_logic_vector(17 downto 0) := (others => '0' );
constant MONITOR_PORT : std_logic_vector(7 downto 0) := (others => '1' );       -- port_id 0xFF

signal pb_address : std_logic_vector(imem_a_width-1 downto 0);
signal pb_instruction : std_logic_vector(17 downto 0);
signal pb_bram_enable : std_logic;          -- kcpsm6
signal pb_port_id : std_logic_vector(7 downto 0);
signal pb_write_strobe : std_logic;
signal pb_k_write_strobe : std_logic;       -- kcpsm6
signal pb_out_port : std_logic_vector(7 downto 0);
signal pb_read_strobe : std_logic;
signal pb_port_busy   : std_logic;
signal pb_in_port : std_logic_vector(7 downto 0);
signal pb_interrupt : std_logic;
signal pb_interrupt_ack : std_logic;
signal pb_sleep : std_logic;                -- kcpsm6

    --! debug ports
signal pb_dbg_rslt_we : std_logic;                                        --! writing result into rX
signal pb_dbg_rslt_dt : std_logic_vector(PB3_REGISTER_WIDTH-1 downto 0);  --! the result value

signal reset : std_logic;
signal clk : std_logic;
signal simdone : std_logic;

signal cycle : integer;         -- debug: picoblaze cycle number

begin
    reset <= '1', '0' after 55 ns;
    
-- clock generator
    clkgen: process
    begin
        clk <= '1';
        wait for 5 ns;
        clk <= '0';
        wait for 5 ns;
        if simdone = '1' then
            wait;
        end if;
    end process;
    
    arch_kcpsm3: if pbver = "kcpsm3" generate
        i_pb_kcpsm3: kcpsm3
        port map (
            pb_address, pb_instruction,
            pb_port_id, pb_write_strobe, pb_out_port, pb_read_strobe, pb_in_port,
            pb_interrupt, pb_interrupt_ack,
            reset, clk
        );
        pb_k_write_strobe <= '0';
    end generate;
    
    arch_pb3a: if pbver = "pb3a" generate
        i_pb_kcpsm3: pacoblaze3a
        port map (
            pb_address, pb_instruction,
            pb_port_id, pb_write_strobe, pb_out_port, pb_read_strobe, pb_in_port, pb_port_busy,
            pb_interrupt, pb_interrupt_ack,
            reset, clk,
            -- output debug ports:
            pb_dbg_rslt_we, pb_dbg_rslt_dt
        );
        pb_k_write_strobe <= '0';
    end generate;

    arch_kcpsm6: if pbver = "kcpsm6" generate
        i_pb_kcpsm6: kcpsm6
        port map (
            pb_address, pb_instruction, pb_bram_enable,
            pb_in_port, pb_out_port, pb_port_id, pb_write_strobe, pb_k_write_strobe, 
            pb_read_strobe, pb_interrupt, pb_interrupt_ack,
            pb_sleep, reset, clk
        );
    end generate;

    pb_port_busy <= '0';        -- not used
    pb_interrupt <= '0';
    pb_in_port <= (others => '0');
    pb_sleep <= '0';

-- instruction memory
    imemsim: process
    variable imem: mem_id_type;
    variable instr: std_logic_vector(17 downto 0);
    begin
        imem := SRAM_Initialize("imem", 2**imem_a_width, 18, zero_instruction);
        Mem_Load(imem, imem_fname);
        
        loop
            wait until (clk = '1' and reset = '0');               -- rising edge
            MEM_Read(imem, pb_address, instr);
            pb_instruction <= instr;
        end loop;
    end process;
    
-- monitor
    monitor: process
    begin
        simdone <= 'L';
        loop
            wait until (clk = '1' and reset = '0');               -- rising edge
            -- sample on rising edge
            if (pb_write_strobe = '1') and (pb_port_id = MONITOR_PORT) then
                --if (pb_out_port = "11111111") then
                simdone <= '1';     -- force end-of-sim
                printf("cyc: %s, monitor-process, request-to-stop", integer'image(cycle));
                --end if;
            end if;
        end loop;
        wait;
    end process;

    prnoutp: process
    file prtdump : ascii_text open write_mode is portmon_fname;
    variable strbuf : string(1 to MAXSTRLEN);
    begin
        loop
            wait until (clk = '1' and reset = '0');               -- rising edge
            -- sample on rising edge
            if (pb_write_strobe = '1') then
                printf("cyc: %s, port-write, port_id: 0x%s, data: 0x%s",
                    integer'image(cycle-1), tosth(pb_port_id), tosth(pb_out_port));
                fprint(strbuf, "%s  W  0x%s  0x%s  %s",
                    integer'image(cycle-1), tosth(pb_port_id), tosth(pb_out_port), 
                    character'image(character'val( to_integer(unsigned(pb_out_port)))) );
                fputs(strbuf, prtdump);
            end if;
            if (pb_k_write_strobe = '1') then
                printf("cyc: %s, port-k-write, port_id: 0x%s, data: 0x%s",
                    integer'image(cycle-1), tosth(pb_port_id), tosth(pb_out_port));
                fprint(strbuf, "%s  W  0x%s  0x%s  %s",
                    integer'image(cycle-1), tosth(pb_port_id), tosth(pb_out_port), 
                    character'image(character'val( to_integer(unsigned(pb_out_port)))) );
                fputs(strbuf, prtdump);
            end if;
            if (pb_read_strobe = '1') then
                printf("cyc: %s, port-read, port_id: 0x%s, data: 0x%s",
                    integer'image(cycle-1), tosth(pb_port_id), tosth(pb_in_port));
                fprint(strbuf, "%s  R  0x%s  0x%s  %s",
                    integer'image(cycle-1), tosth(pb_port_id), tosth(pb_in_port),
                    character'image(character'val( to_integer(unsigned(pb_out_port)))) );
                fputs(strbuf, prtdump);
            end if;
        end loop;
    end process;

-- disassemble picoblaze instructions
    pb_disas: process
    variable last_ins : std_logic_vector(17 downto 0);
    variable addr, last_addr : std_logic_vector(imem_a_width-1 downto 0);
    variable dcd : string(1 to 100);
    variable rslt : string(1 to 50);
    constant space : string(1 to 80) := "                                                                                ";
    variable spbuf : string(1 to 80);
    variable istr: string(1 to MAXSTRLEN);
    begin
        cycle <= 0;
        loop
            wait until (clk = '1' and reset = '0');               -- rising edge
            addr := pb_address;         -- sample on rising edge
            wait for 1 ns;              -- let the instruction code propagate
            
            if (last_addr /= addr) or (last_ins /= pb_instruction) then
                if (pbver = "kcpsm3") or (pbver = "pb3a") then
                    istr := pb3instr2string(pb_instruction);
                elsif (pbver = "kcpsm6") then
                    istr := pb6instr2string(pb_instruction);
                end if;
                fprint(dcd, "cyc: %s, PC 0x%s, op: 0x%s  %s",
                       integer'image(cycle), tosth(addr), tosth(pb_instruction), istr);
            else
                if (pb_dbg_rslt_we = '1') then
                    -- std_logic'image(pb_dbg_rslt_we), 
                    fprint(rslt, "; s%s <- 0x%s", tosth(pb_instruction(11 downto 8)), tosth(pb_dbg_rslt_dt));
                    strncpy(spbuf, space, 60-strlen(dcd));
                    printf("%s%s%s", dcd, spbuf, rslt);
                else
                    rslt(1) := NUL;
                    printf("%s", dcd);
                end if;
            end if;
            
            last_addr := addr;
            last_ins := pb_instruction;
            cycle <= cycle + 1;
        end loop;
    end process;

end architecture behav;
